# Copyright 2024 UW-IT, University of Washington
# SPDX-License-Identifier: Apache-2.0

import os
from urllib3 import PoolManager
from urllib3.util.retry import Retry
from os.path import abspath, dirname
from restclients_core.dao import DAO


class MSCA_DAO(DAO):
    def service_name(self):
        return 'msca'

    def service_mock_paths(self):
        return [abspath(os.path.join(dirname(__file__), "resources"))]

    def _custom_headers(self, method, url, headers, body):
        return {"Ocp-Apim-Subscription-Key": "{}".format(
            self.get_service_setting("SUBSCRIPTION_KEY", ""))}

    def getURL(self, url, headers={}, body=None):
        return self._load_resource("GET", url, headers, body)

    def get_external_resource(self, url, body=None):
        http = PoolManager(
            retries=Retry(total=1, connect=0, read=0, redirect=1))
        return http.request('GET', url, body=body)



# Copyright 2024 UW-IT, University of Washington
# SPDX-License-Identifier: Apache-2.0

"""
This is the interface for interacting with
the uwnetid subscription web service.
"""

import logging
from commonconf import settings
from restclients_core.exceptions import DataFailureException
from uw_msca.dao import MSCA_DAO


DAO = MSCA_DAO()
logger = logging.getLogger(__name__)


def url_base(override=None):
    # until msca api finalized
    return '/{}/{}'.format(
        override if override else 'mbx',
        getattr(settings, 'RESTCLIENTS_MSCA_VERSION', 'v1'))


def get_resource(url, headers=None):
    default_headers = {"Accept": "application/json"}
    if headers:
        default_headers.update(headers)

    response = DAO.getURL(url, default_headers)
    logger.debug("GET {0} ==status==> {1}".format(url, response.status))
    if response.status != 200:
        raise DataFailureException(url, response.status, response.data)

    logger.debug("GET {0} ==data==> {1}".format(url, response.data))

    return response.data


def post_resource(url, body):
    response = DAO.postURL(url, {
        'Content-Type': 'application/json',
        'Acept': 'application/json',
    }, body)
    logger.debug("POST {0} ==status==> {1}".format(url, response.status))

    if response.status != 200:
        raise DataFailureException(url, response.status, response.data)

    logger.debug("POST {0}s ==data==> {1}".format(url, response.data))

    return response.data


def put_resource(url, body, headers=None):
    default_headers = {
        "Accept": "application/json",
        "Content-Type": "application/json",
    }
    if headers:
        default_headers.update(headers)

    response = DAO.putURL(
        url,
        default_headers,
        body,
    )
    logger.debug("PUT {0} ==status==> {1}".format(url, response.status))

    if response.status != 200:
        raise DataFailureException(url, response.status, response.data)

    logger.debug("PUT {0}s ==data==> {1}".format(url, response.data))

    return response.data


def patch_resource(url, body):
    response = DAO.patchURL(url, {
        'Content-Type': 'application/json',
        'Acept': 'application/json',
    }, body)
    logger.debug("PATCH {0} ==status==> {1}".format(url, response.status))

    if response.status != 200:
        raise DataFailureException(url, response.status, response.data)

    logger.debug("PATCH {0}s ==data==> {1}".format(url, response.data))

    return response.data


def get_external_resource(url, body=None):
    response = DAO.get_external_resource(url, body=body)

    logger.debug(
        "external_resource {0} ==status==> {1}".format(url, response.status))

    if response.status != 200:
        raise DataFailureException(url, response.status, response.data)

    logger.debug(
        "external_resource {0}s ==data==> {1}".format(url, response.data))

    return response.data



# Copyright 2024 UW-IT, University of Washington
# SPDX-License-Identifier: Apache-2.0

from unittest import TestCase
from uw_msca.validate_user import validate_user
from uw_msca.util import fdao_msca_override


@fdao_msca_override
class ValidateUserTest(TestCase):
    def test_valid_user(self):
        validated = validate_user('javerage')
        self.assertTrue(validated.valid)

    def test_invalid_user(self):
        validated = validate_user('bill')
        self.assertFalse(validated.valid)



# Copyright 2024 UW-IT, University of Washington
# SPDX-License-Identifier: Apache-2.0

from unittest import TestCase
from unittest.mock import (
    patch,
)

from commonconf import override_settings

from uw_msca.shared_drive import (
    DAO,
    GoogleDriveState,
    get_default_org_unit,
    get_default_quota,
    get_google_drive_states,
    set_drive_quota,
    _msca_drive_base_url,
)


@override_settings(
    RESTCLIENTS_MSCA_OAUTH_TOKEN_URL=(
        "https://login.microsoftonline.com/example.uw.edu/oauth2/v2.0/token",
    ),
    RESTCLIENTS_MSCA_REPORT_SCOPE=(
        "api://beef821f-dead-46ac-9829-f9a87eb12c37/.default"
    ),
    RESTCLIENTS_MSCA_CLIENT_ID="beef821f-dead-46ac-9829-f9a87eb12c37",
    RESTCLIENTS_MSCA_CLIENT_SECRET="my-secret",
    RESTCLIENTS_MSCA_HOST="https://msca.hosts",
    RESTCLIENTS_MSCA_SUBSCRIPTION_KEY="my-subscription-key",
    RESTCLIENTS_MSCA_DAO_CLASS="Mock",
)
class BaseGDriveTest(TestCase):
    "Base class for GDrive tests."
    abstract = True


class Test_MSCA_GDrive(BaseGDriveTest):
    def test_msca_drive_base_url(self):
        assert _msca_drive_base_url() == "/google/v1/drive"


class Test_get_default_org_unit(BaseGDriveTest):
    def test(self):
        with patch.object(
            DAO,
            "get_external_resource",
            side_effect=[
                # returned after first call
                DAO.getURL("/google/token_response_fixture"),
                # further calls raise a StopIteration
            ],
        ):
            result = get_default_org_unit()
        assert result == "100GB"


class Test_get_default_quota(BaseGDriveTest):
    def test(self):
        with patch.object(
            DAO,
            "get_external_resource",
            side_effect=[
                # returned after first call
                DAO.getURL("/google/token_response_fixture"),
                # further calls raise a StopIteration
            ],
        ):
            result = get_default_quota()
        assert result == 100


class Test_get_google_drive_states(BaseGDriveTest):
    def test(self):
        with patch.object(
            DAO,
            "get_external_resource",
            side_effect=[
                # returned after first call
                DAO.getURL("/google/token_response_fixture"),
                # returned after second call
                DAO.getURL("/google/report_response_fixture"),
                # further calls raise a StopIteration
            ],
        ):

            gdrive_states = get_google_drive_states()

        assert len(gdrive_states) == 3
        assert all(isinstance(X, GoogleDriveState) for X in gdrive_states)
        assert gdrive_states[0].org_unit_name == "uw.edu"
        assert gdrive_states[0].org_unit_id == "00gjdgxs0123458"
        assert gdrive_states[0].total_uw_owners == 2
        assert gdrive_states[0].size == 307


class Test_set_drive_quota(BaseGDriveTest):
    def test(self):
        drive_id = "0AIdwn8Py42DEADBEEF"
        quota = 3000
        with patch.object(
            DAO,
            "get_external_resource",
            side_effect=[
                DAO.getURL("/google/token_response_fixture"),
            ],
        ):
            result = set_drive_quota(quota=quota, drive_id=drive_id)

        assert result == {
            "message": f"Drive '{drive_id}' successfully moved to 3000GB"
        }



# Copyright 2024 UW-IT, University of Washington
# SPDX-License-Identifier: Apache-2.0

from unittest import TestCase
from uw_msca.delegate import get_delegates
from uw_msca.util import fdao_msca_override


@fdao_msca_override
class ValidateUserTest(TestCase):
    def test_get_delegates(self):
        delegates = get_delegates('javerage')
        self.assertEqual(len(delegates), 2)

        delegates = get_delegates('bill')
        self.assertEqual(len(delegates), 1)



# Copyright 2024 UW-IT, University of Washington
# SPDX-License-Identifier: Apache-2.0

from unittest import TestCase

from uw_msca.models import (
    Quota,
)


class Test_Quota(TestCase):
    def test_to_str(self):
        assert Quota.to_str(100) == "100GB"
        assert Quota.to_str(1000) == "1000GB"
        assert Quota.to_str(9000) == "9000GB"

    def test_to_int(self):
        assert Quota.to_int("100GB") == 100
        assert Quota.to_int("1000GB") == 1000
        assert Quota.to_int("15000GB") == 15000

    def test_to_int_errors(self):
        with self.assertRaises(ValueError):
            Quota.to_int("100")



# Copyright 2024 UW-IT, University of Washington
# SPDX-License-Identifier: Apache-2.0

import json
from restclients_core import models


class ValidatedUser(models.Model):
    name = models.SlugField(max_length=32)
    valid = models.NullBooleanField()

    def is_valid(self):
        return self.valid

    def from_json(self, data):
        self.name = data["name"]
        self.valid = self._set_validity(data.get("valid"))
        return self

    def _set_validity(self, valid):
        """
        accept a range of truthy values and return a bool
        """
        return (
            valid
            if isinstance(valid, bool)
            else (
                (valid.lower() in ["true", "yes", "1"])
                if (isinstance(valid, str))
                else (valid != 0) if (isinstance(valid, int)) else False
            )
        )

    def json_data(self):
        return {"name": self.name, "valid": self.valid}

    def __str__(self):
        return json.dumps(self.json_data())


class Delegate(models.Model):
    user = models.SlugField(max_length=32)
    delegate = models.SlugField(max_length=32)
    access_right = models.SlugField(max_length=32)

    def from_json(self, user, data):
        self.user = user
        self.delegate = data["User"]
        self.access_right = data["AccessRights"]
        return self

    def json_data(self):
        return {
            "user": self.user,
            "delegate": self.name,
            "access_right": self.access_right,
        }

    def __str__(self):
        return json.dumps(self.json_data())


class AccessRight(models.Model):
    right_id = models.SmallIntegerField()
    displayname = models.SlugField(max_length=32)

    def from_json(self, data):
        self.right_id = data.get("id")
        self.displayname = data.get("displayname")
        return self

    def json_data(self):
        return {"displayname": self.displayname, "id": self.right_id}

    def __str__(self):
        return json.dumps(self.json_data())


class GoogleDriveState(models.Model):
    # max_length values informed by examining all current results
    # TODO: more thorough solution
    drive_id = models.SlugField(max_length=19)
    drive_name = models.SlugField(max_length=125)
    total_members = models.PositiveIntegerField()
    org_unit_id = models.SlugField(max_length=15)
    # e.g., "100 TB"; longest observed as of 2024-04-03 is 12 long
    org_unit_name = models.SlugField(max_length=20)
    member = models.SlugField(max_length=66)
    role = models.CharField(max_length=13)
    query_date = models.DateTimeField()
    total_uw_owners = models.PositiveIntegerField()
    size = models.PositiveIntegerField()

    EXPECTED_CSV_FIELDS = (
        "id",
        "drive_id",
        "drive_name",
        "member",
        "role",
        "total_members",
        "total_uwowners",
        "org_unitID",
        "org_unitName",
        "query_date",
        "size",
        "file_count",
        "size_query_date",
    )

    CSV_FIELD_MAP = {
        "org_unitID": "org_unit_id",
        "org_unitName": "org_unit_name",
        "total_uwowners": "total_uw_owners"
    }

    @property
    def quota_limit(self):
        return Quota.to_int(self.org_unit_name)

    @property
    def size_gigabytes(self):
        """
        Return size in GB.

        Native size field is in MB.
        """
        return self.size / 1024

    @classmethod
    def from_csv(cls, csv_data: dict):
        """
        Factory for creating from CSV data from a csv.DictReader.
        """
        fields = dict([cls._map(k, v) for (k, v) in csv_data.items()])
        return cls(**fields)

    @classmethod
    def _map(cls, k, v):
        """
        Map CSV field name to model field name
        and CSV string value to appropriate model class value
        """
        if k not in cls.EXPECTED_CSV_FIELDS:
            raise ValueError(f"Unexpected field: {k}")

        try:
            field_name = cls.CSV_FIELD_MAP[k]
        except KeyError:
            field_name = k

        if 'Integer' in cls._field_class(field_name):
            try:
                field_value = int(v)
            except ValueError:
                field_value = 0
        else:
            field_value = v

        return field_name, field_value

    @classmethod
    def _field_class(cls, field_name):
        """
        Return the class name for the provided field name
        """
        try:
            return cls.__dict__[field_name].__class__.__name__
        except KeyError:
            return ''

    @classmethod
    def _int(cls, value):
        try:
            return int(value)
        except ValueError:
            return 0


class Quota:
    """
    Quota translation class.

    Quotas are implemented in the MSCA backend via Organization Units.

    Reporting from MSCA refers to this org unit instead so conversions must be
    easy.
    """

    # TODO: handle the "queued for deletion" OU name

    def to_str(quota: int):
        "Convert int quota from PRT to str quota MSCA uses internally."
        # TODO: type check / error?!
        return f"{quota}GB"

    def to_int(quota: str):
        "Convert a str quota for MSCA internal use to an int PRT can use."
        if not quota.endswith("GB"):
            raise ValueError(f"String {quota} does not end in 'GB'.")

        try:
            return int(quota[:-2])
        except ValueError:
            s = quota[:-2]
            if not s.isdigit():
                raise ValueError(f"Quota {quota} is not digits + 'GB' suffix")
            else:
                raise ValueError(f"Quota {quota} failed to convert {s} to int")



# Copyright 2024 UW-IT, University of Washington
# SPDX-License-Identifier: Apache-2.0

"""
Interface for interacting with UW MSCA outlook API
"""

import logging
import json
from uw_msca.models import AccessRight
from uw_msca import url_base, get_resource


logger = logging.getLogger(__name__)


def get_access_rights():
    """
    Returns list of Outlook mailbox Access Rights
    """
    url = _msca_access_rights_url()
    response = get_resource(url)
    return _json_to_supported(response)


def _msca_access_rights_url():
    """
    Return UW MSCA uri for Office access right resource
    """
    return "{}/GetAccessRights".format(url_base())


def _json_to_supported(response_body):
    """
    Returns a list of Supported objects
    """
    data = json.loads(response_body)
    access_rights = []

    for access_right in data.get('value'):
        access_rights.append(AccessRight().from_json(access_right))

    return access_rights



# Copyright 2024 UW-IT, University of Washington
# SPDX-License-Identifier: Apache-2.0

from unittest import TestCase
from uw_msca.access_rights import get_access_rights
from uw_msca.util import fdao_msca_override


@fdao_msca_override
class AccessRightsTest(TestCase):
    def test_get_access_rights(self):
        access_rights = get_access_rights()
        self.assertEqual(len(access_rights), 4)



# Copyright 2024 UW-IT, University of Washington
# SPDX-License-Identifier: Apache-2.0

from restclients_core.util.decorators import use_mock
from uw_msca.dao import MSCA_DAO


fdao_msca_override = use_mock(MSCA_DAO())



# Copyright 2024 UW-IT, University of Washington
# SPDX-License-Identifier: Apache-2.0

"""
Interface for interacting with the UW MSCA outlook API
"""

from uw_msca.models import Delegate
from uw_msca import (url_base, get_resource, post_resource,
                     patch_resource, get_external_resource)
import json
import logging


logger = logging.getLogger(__name__)


def _delegate_url_base(netid):
    """
    Return UW MSCA base uri for Office access delegates
    """
    return "{}/{}".format(url_base(), netid)


def _msca_get_delegate_url(netid):
    """
    Return UW MSCA uri for Office access delegates
    """
    return "{}/GetDelegates".format(_delegate_url_base(netid))


def _msca_get_all_delegates_csv_url():
    """
    Return UW MSCA uri for Office all access delegates
    """
    return "{}/GetDelegateCsv".format(url_base())


def _msca_set_delegate_url(netid, delegate, access_type):
    """
    Return UW MSCA uri to set delegate access
    """
    return "{}/SetDelegatePerms/{}/{}".format(
        _delegate_url_base(netid), delegate, access_type)


def _msca_update_delegate_url(netid, delegate, old_access_type, access_type):
    """
    Return UW MSCA uri to set delegate access
    """
    return "{}/UpdateDelegatePerms-azf/{}/{}/{}".format(
        _delegate_url_base(netid), delegate, old_access_type, access_type)


def _msca_remove_delegate_url(netid, delegate, access_type):
    """
    Return UW MSCA uri for removing delegate access
    """
    return "{}/RemoveDelegatePerms/{}/{}".format(
        _delegate_url_base(netid), delegate, access_type)


def get_delegates(netid):
    """
    Returns whether or not given user has access to Outlook mailbox
    """
    url = _msca_get_delegate_url(netid)
    response = get_resource(url)

    json_response = json.loads(response)
    user = json_response['TargetNetid']
    delegates = []
    for delegate in json_response['Delegates']:
        delegates.append(Delegate().from_json(user, delegate))

    return delegates


def get_all_delegates():
    """
    method returns all delegations assigned in outlook via
    two sequence request.  first, request url for all delegate csv.
    second, request csv from url returned in first request.
    from: https://pplat-apimgmt.azure-api.net/mbx/v1/GetDelegateCsv
    """
    delegates_csv_url = _msca_get_all_delegates_csv_url()
    csv_url = get_resource(delegates_csv_url).decode('utf-8')
    response = get_external_resource(csv_url)
    return response.decode('utf-8').split('\r\n')


def set_delegate(netid, delegate, access_type):
    """
    Returns with delegate access set for netid resource
    """
    url = _msca_set_delegate_url(netid, delegate, access_type)
    body = json.dumps({
        'netid': netid,
        'delegate': delegate,
        'accesstype': access_type
    })

    response = post_resource(url, body)

    delegates = []
    try:
        json_response = json.loads(response)
        user = json_response['TargetNetid']
        for delegate in json_response['Delegates']:
            delegates.append(Delegate().from_json(user, delegate))
    except Exception as ex:
        logger.error("set_delegate response: -->{}<-- error: {}".format(
            response, ex))

    return delegates


def update_delegate(netid, delegate, old_access_type, new_access_type):
    """
    Returns with delegate access set for netid resource
    """
    url = _msca_update_delegate_url(
        netid, delegate, old_access_type, new_access_type)
    body = json.dumps({
        'netid': netid,
        'delegate': delegate,
        'RemoveAccesstype': old_access_type,
        'SetAccesstype': new_access_type,
    })

    response = patch_resource(url, body)

    delegates = []
    try:
        json_response = json.loads(response)
        user = json_response['TargetNetid']
        for delegate in json_response['Delegates']:
            delegates.append(Delegate().from_json(user, delegate))
    except Exception as ex:
        logger.error("set_delegate response: -->{}<-- error: {}".format(
            response, ex))

    return delegates


def remove_delegate(netid, delegate, access_type):
    """
    Returns with delegate access removed from netid resource
    """
    url = _msca_remove_delegate_url(netid, delegate, access_type)
    body = json.dumps({
        'netid': netid,
        'delegate': delegate,
        'accesstype': access_type
    })

    response = post_resource(url, body)

    delegates = []
    try:
        json_response = json.loads(response)
        user = json_response['TargetNetid']
        for delegate in json_response['Delegates']:
            delegates.append(Delegate().from_json(user, delegate))
    except Exception as ex:
        logger.error("remove_delegate response: -->{}<-- error: {}".format(
            response, ex))

    return delegates



# Copyright 2023 UW-IT, University of Washington
# SPDX-License-Identifier: Apache-2.0

import os
from setuptools import setup

README = """
See the README on `GitHub
<https://github.com/uw-it-aca/uw-restclients-msca>`_.
"""

version_path = 'uw_msca/VERSION'
VERSION = open(os.path.join(os.path.dirname(__file__), version_path)).read()
VERSION = VERSION.replace("\n", "")

# allow setup.py to be run from any path
os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))

setup(
    name='UW-RestClients-MSCA',
    version=VERSION,
    packages=['uw_msca'],
    author="UW-IT AXDD",
    author_email="aca-it@uw.edu",
    include_package_data=True,
    install_requires=['UW-RestClients-Core~=1.3',
                      ],
    license='Apache License, Version 2.0',
    description=('A library for connecting to the UW NetID API'),
    long_description=README,
    url="https://github.com/uw-it-aca/uw-restclients-msca",
    classifiers=[
        'Intended Audience :: Developers',
        'License :: OSI Approved :: Apache Software License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3.6',
    ],
)



# Copyright 2024 UW-IT, University of Washington
# SPDX-License-Identifier: Apache-2.0
"""
This file contains the interfaces for MSCA's /google/vN/drive endpoints.

Of note, maximum drive disk quota is not well supported by Google. At this time
of writing support was provided only via the Organization Unit (OU) the drive
is located in.
"""

import csv
import io
import json
import logging
from urllib.parse import urlencode

from uw_msca import (
    DAO,
    url_base,
    get_resource,
    get_external_resource,
    put_resource,
)

from uw_msca.models import (
    GoogleDriveState,
    Quota,
)


def get_default_org_unit():
    """
    Return the default/subsidized Org Unit for Shared Drives.
    """
    org_unit_resp = get_resource(
        url=_get_default_org_unit_url(),
        headers=_authorization_headers(),
    )
    j = json.loads(org_unit_resp)
    return j["ou"]


def get_default_quota():
    default_quota: str = get_default_org_unit()
    return Quota.to_int(default_quota)


def get_google_drive_states():
    """
    Return list of GoogleDriveState's from report generated by PPLAT.
    """
    drive_state_reports_resp = get_resource(
        url=_get_drivestate_url(),
        headers=_authorization_headers(),
    )

    drive_state_reports_url = json.loads(drive_state_reports_resp)["sasKey"]
    report_resp: bytes = get_external_resource(drive_state_reports_url)

    # wrap encoded bytes such that they can be iterated over per-line
    filelike = io.TextIOWrapper(io.BytesIO(report_resp))

    records = csv.DictReader(filelike)
    if not set(records.fieldnames).issuperset(
        GoogleDriveState.EXPECTED_CSV_FIELDS
    ):
        missing = [
            X
            for X in GoogleDriveState.EXPECTED_CSV_FIELDS
            if X not in records.fieldnames
        ]
        logging.error(
            f"Missing expected fields from {_get_drivestate_url()}: {missing}"
        )

    result = []

    for record in records:
        result.append(GoogleDriveState.from_csv(record))

    return result


def set_drive_quota(quota: int, drive_id: str):
    """
    Update Google Drive to have the specified quota.

    Args:
        quota: integer quota in units of GB. 100 == 100GB

    Raises:
        ValueError: quota is non-integer value.
    """
    str_quota = Quota.to_str(quota)

    # in this case quota values are implicitly provided by the
    # Organizational Unit (OU) of the drive
    # so what we're really doing is moving a drive between OUs
    data = {"quota": str_quota}

    resp_data = put_resource(
        url=_set_quota_url(drive_id),
        body=json.dumps(data),
        headers=_authorization_headers(),
    )

    try:
        # Mike asked for a JSON payload
        return json.loads(resp_data)
    except json.JSONDecodeError:
        # until Ken delivers change
        result = resp_data.decode()
        return {"message": result}


def _set_quota_url(drive_id):
    return f"{_msca_drive_base_url()}/{drive_id}/setquota"


def _get_default_org_unit_url():
    return f"{_msca_drive_base_url()}/defaultou"


def _get_drivestate_url():
    return f"{_msca_drive_base_url()}/getfile"


def _msca_drive_base_url():  # returns "/google/v1/drive"
    base = url_base(override="google")
    return f"{base}/drive"


def _authorization_headers():
    """
    Get OAuth bearer token for authorization in subsequent requests.
    """
    data = {
        "client_id": DAO.get_service_setting("CLIENT_ID"),
        "client_secret": DAO.get_service_setting("CLIENT_SECRET"),
        "grant_type": "client_credentials",
        "scope": DAO.get_service_setting("REPORT_SCOPE"),
    }
    body = urlencode(data)

    token_response = get_external_resource(
        DAO.get_service_setting("OAUTH_TOKEN_URL"),
        body=body,
    )
    token = json.loads(token_response)["access_token"]

    auth_headers = {"Authorization": f"Bearer {token}"}

    return auth_headers



# Copyright 2024 UW-IT, University of Washington
# SPDX-License-Identifier: Apache-2.0

"""
Interface for interacting with the UW MSCA outlook API
"""

import logging
import json
from uw_msca.models import ValidatedUser
from uw_msca import url_base, get_resource


logger = logging.getLogger(__name__)


def validate_user(name):
    """
    Returns whether or not given user has access to Outlook mailbox
    """
    url = _msca_validate_user_url(name)
    response = get_resource(url)
    return ValidatedUser().from_json(json.loads(response))


def _msca_validate_user_url(name):
    """
    Return UW MSCA uri for Office access validation
    """
    return "{}/ValidateUser?Name={}".format(url_base(), name)
